//! Quick and dirty program to build all shaders and print any errors.
//!
//! Typical use of shaders is just to `include_str!()` the source, but this
//! makes debugging compile errors in shaders tedious; you have to rebuild
//! your whole Rust program every time.
//! This program is hopefully a solution.

extern crate engine;

use std::fs;
use std::path::PathBuf;
use engine::*;

fn main() {
	// Parse arguments.
	let mut iter = std::env::args();
	let binary = iter.next().unwrap_or_else(|| String::from("program"));
	let dir_maybe = iter.next();
	if dir_maybe.is_none() || iter.next().is_some() {
		println!("Usage: {} directory", binary);
		println!("Compiles all .vert and .frag files found in directory as GLSL and prints resulting errors");
		return;
	}
	let dir = dir_maybe.unwrap();

	// Create window so we have a GL context.
	let mut window = Window::new("shader check").unwrap();

	// Find and compile shaders.
	for entry in fs::read_dir(dir).unwrap() {
		let entry = entry.unwrap();
		let mut base_path = entry.path();
		let (is_vert, is_frag) = match base_path.extension() {
			Some(extension) => (extension == "vert", extension == "frag"),
			None => (false, false),
		};
		assert!(base_path.set_extension(""));
		let name = base_path.file_name().unwrap().to_string_lossy().into_owned();
		if is_vert {
			check(&mut window, &name, base_path);
		} else if is_frag {
			assert!(base_path.set_extension("vert"));
			if !base_path.is_file() {
				println!("[err] {}: couldn't read vertex shader", name);
			}
		} else {
			println!("[skip] {}", name);
		}
	}
}

fn check(window: &mut Window, name: &str, mut base_path: PathBuf) {
	assert!(base_path.set_extension("vert"));
	let vertex = match fs::read_to_string(&base_path) {
		Ok(data) => data,
		Err(error) => return println!("[err] {}: couldn't read vertex shader ({})", name, error),
	};
	assert!(base_path.set_extension("frag"));
	let fragment = match fs::read_to_string(&base_path) {
		Ok(data) => data,
		Err(error) => return println!("[err] {}: couldn't read fragment shader ({})", name, error),
	};
	if let Err(error) = Shader::<Vertex>::new(window, "(unnamed)", &vertex, &fragment) {
		return println!("[err] {}: {}", name, error);
	}
	println!("[ok] {}", name);
}
