//! This file is part of Flight.
//! 
//! Flight is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//! 
//! Flight is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//! You should have received a copy of the GNU General Public License along with Flight. If not, see <https://www.gnu.org/licenses/>.

use super::ffi_keycodes::SDL_KeyCode;
use super::vector2::F2;

#[derive(Clone, Copy, Debug, PartialEq, Eq)]
pub enum Button { Left, Middle, Right }

#[derive(Clone, Copy, Debug, PartialEq, Eq)]
#[repr(u32)]
pub enum ControllerAxis { LeftTrigger, LeftX, LeftY, RightTrigger, RightX, RightY }
pub const CONTROLLER_AXIS_COUNT: usize = ControllerAxis::RightY as usize+1;

#[derive(Clone, Copy, Debug, PartialEq, Eq)]
#[repr(u32)]
pub enum ControllerButton {
	DpadLeft, DpadRight, DpadUp, DpadDown,
	A, B, X, Y,
	LeftShoulder, RightShoulder,
	LeftStick, RightStick,
	Start, Back,
}
pub const CONTROLLER_BUTTON_COUNT: usize = ControllerButton::Back as usize+1;

pub type Key = SDL_KeyCode;

#[derive(Clone, Copy, Debug, PartialEq, Eq)]
pub struct Modifiers {
	pub alt: bool,
	pub ctrl: bool,
	pub shift: bool,
}

#[derive(Clone, Copy, Debug, PartialEq)]
pub enum Event {
	Button{button: Button, down: bool, position: F2},
	ControllerAdded{id: u32},
	ControllerAxis{id: u32, axis: ControllerAxis, value: f32},
	ControllerButton{id: u32, button: ControllerButton, down: bool},
	ControllerLost{id: u32},
	Key{key: Key, down: bool, modifiers: Modifiers, repeat: bool},
	Motion{position: F2, relative: F2},
	Quit,
	Resize,
	Wheel{x: i32, y: i32},
}
