//! This file is part of Flight.
//! 
//! Flight is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//! 
//! Flight is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//! You should have received a copy of the GNU General Public License along with Flight. If not, see <https://www.gnu.org/licenses/>.

// Return offset of a field in a structure.
// Usage:
// ```
// struct Data {
//     x: i32,
//     y: i32,
// }
// assert_eq!(offset_of!(Data, y), 4);
// ```
#[macro_export]
macro_rules! offset_of {
	($struct:ident, $field:ident) => {{
		let value: $struct = std::mem::MaybeUninit::zeroed().assume_init();
		let start = &value as *const $struct;
		let field;
		#[cfg(raw_ref_macros)]
		{
			// Avoid undefined behaviour via Rust 1.51 feature
			field = std::ptr::addr_of!(value.$field);
		}
		#[cfg(not(raw_ref_macros))]
		{
			// Fallback for older compilers
			field = &value.$field as *const _;
		}
		field as usize-start as usize
	}}
}
