//! This file is part of Flight.
//! 
//! Flight is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//! 
//! Flight is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//! You should have received a copy of the GNU General Public License along with Flight. If not, see <https://www.gnu.org/licenses/>.

/// Simple, fast, *non*crptographic random number generator.
/// Automatically seeded from system clock.
/// Randomness and uniformity of random distribution is sacrificed for speed in some cases.
#[derive(Clone)]
pub struct RNG {
	state: u64,
}
impl RNG {
	pub fn new() -> Self {
		use std::time::SystemTime;
		Self {
			state: SystemTime::now().duration_since(SystemTime::UNIX_EPOCH).unwrap().as_millis() as u64,
		}
	}
	pub fn next(&mut self) -> u64 {
		let mut value = self.state;
		value ^= value << 13;
		value ^= value >> 7;
		value ^= value << 17;
		self.state = value;
		return value;
	}
	pub fn bool(&mut self) -> bool {
		self.next()&1 != 0
	}
	pub fn u8(&mut self) -> u8 {
		self.next() as u8
	}
	pub fn u32(&mut self) -> u32 {
		self.next() as u32
	}
	pub fn f32(&mut self) -> f32 { // 0..1
		(self.next()&0xffffffffu64) as f32/0xffffffffu64 as f32
	}
	pub fn choose<'a, T>(&mut self, items: &'a [T]) -> &'a T {
		&items[self.next() as usize%items.len()]
	}
}
