precision mediump float;

uniform sampler2D texture_heightmap;
uniform sampler2D texture_detail;
uniform sampler2D texture_noise;
uniform float heightmap_size;
uniform float sea_sin;
uniform float sea_cos;
uniform float alpha;

varying float vertex_fog;
varying float vertex_sea;
varying vec2 vertex_texcoord;

void main() {
	float pattern_large = texture2D(texture_detail, vertex_texcoord*heightmap_size).r;
	float pattern_tiny = texture2D(texture_detail, vertex_texcoord*(heightmap_size*16.0)).r;

	vec3 land_colour = texture2D(texture_heightmap, vertex_texcoord).rgb;
	vec3 land_alternate = texture2D(texture_heightmap, vec2(vertex_texcoord.x+1.0/heightmap_size, vertex_texcoord.y)).rgb;
	vec3 land_rgb = mix(land_colour, land_alternate, mix(pattern_large, pattern_tiny, 0.5));

	float sea_alpha = clamp(vertex_sea, 0.0, 1.0);
	vec3 offset_xyz = texture2D(texture_noise, vertex_texcoord*8.0).rgb;
	vec2 offset = mix(offset_xyz.xy, offset_xyz.yz, sea_sin*0.5+0.5)
		+mix(offset_xyz.zx, offset_xyz.yy, sea_cos*0.5+0.5);
	float sea_mix = texture2D(texture_detail, vertex_texcoord*32.0+offset).r;
	vec3 sea_rgb = mix(mix(vec3(0.6, 0.9, 0.9), vec3(0.5, 0.6, 0.9), sea_mix),
		mix(land_colour, land_alternate, (offset.x+offset.y)*0.25),
		0.3);

	vec3 rgb = mix(land_rgb, sea_rgb, sea_alpha);
	rgb = mix(rgb, vec3(0.92, 0.92, 0.97), vertex_fog);
	gl_FragColor = vec4(rgb, alpha);
}
